- {{ formErrors.domains }}
+
diff --git a/client/src/app/shared/shared-moderation/batch-domains-modal.component.ts b/client/src/app/shared/shared-moderation/batch-domains-modal.component.ts
index 6edbb6023..20be728f6 100644
--- a/client/src/app/shared/shared-moderation/batch-domains-modal.component.ts
+++ b/client/src/app/shared/shared-moderation/batch-domains-modal.component.ts
@@ -2,7 +2,7 @@ import { Component, EventEmitter, Input, OnInit, Output, ViewChild } from '@angu
import { FormReactive, FormValidatorService } from '@app/shared/shared-forms'
import { NgbModal } from '@ng-bootstrap/ng-bootstrap'
import { NgbModalRef } from '@ng-bootstrap/ng-bootstrap/modal/modal-ref'
-import { DOMAINS_VALIDATOR, getNotEmptyHosts } from '../form-validators/batch-domains-validators'
+import { splitAndGetNotEmpty, UNIQUE_HOSTS_VALIDATOR } from '../form-validators/host-validators'
@Component({
selector: 'my-batch-domains-modal',
@@ -28,7 +28,7 @@ export class BatchDomainsModalComponent extends FormReactive implements OnInit {
if (!this.action) this.action = $localize`Process domains`
this.buildForm({
- domains: DOMAINS_VALIDATOR
+ hosts: UNIQUE_HOSTS_VALIDATOR
})
}
@@ -41,9 +41,7 @@ export class BatchDomainsModalComponent extends FormReactive implements OnInit {
}
submit () {
- this.domains.emit(
- getNotEmptyHosts(this.form.controls['domains'].value)
- )
+ this.domains.emit(splitAndGetNotEmpty(this.form.controls['hosts'].value))
this.form.reset()
this.hide()
}
diff --git a/server/controllers/api/server/follows.ts b/server/controllers/api/server/follows.ts
index e6f4f6b92..cbe6b7e4f 100644
--- a/server/controllers/api/server/follows.ts
+++ b/server/controllers/api/server/follows.ts
@@ -29,6 +29,7 @@ import {
removeFollowingValidator
} from '../../../middlewares/validators'
import { ActorFollowModel } from '../../../models/actor/actor-follow'
+import { ServerFollowCreate } from '@shared/models'
const serverFollowsRouter = express.Router()
serverFollowsRouter.get('/following',
@@ -45,10 +46,10 @@ serverFollowsRouter.post('/following',
ensureUserHasRight(UserRight.MANAGE_SERVER_FOLLOW),
followValidator,
setBodyHostsPort,
- asyncMiddleware(followInstance)
+ asyncMiddleware(addFollow)
)
-serverFollowsRouter.delete('/following/:host',
+serverFollowsRouter.delete('/following/:hostOrHandle',
authenticate,
ensureUserHasRight(UserRight.MANAGE_SERVER_FOLLOW),
asyncMiddleware(removeFollowingValidator),
@@ -125,8 +126,8 @@ async function listFollowers (req: express.Request, res: express.Response) {
return res.json(getFormattedObjects(resultList.data, resultList.total))
}
-async function followInstance (req: express.Request, res: express.Response) {
- const hosts = req.body.hosts as string[]
+async function addFollow (req: express.Request, res: express.Response) {
+ const { hosts, handles } = req.body as ServerFollowCreate
const follower = await getServerActor()
for (const host of hosts) {
@@ -139,6 +140,18 @@ async function followInstance (req: express.Request, res: express.Response) {
JobQueue.Instance.createJob({ type: 'activitypub-follow', payload })
}
+ for (const handle of handles) {
+ const [ name, host ] = handle.split('@')
+
+ const payload = {
+ host,
+ name,
+ followerActorId: follower.id
+ }
+
+ JobQueue.Instance.createJob({ type: 'activitypub-follow', payload })
+ }
+
return res.status(HttpStatusCode.NO_CONTENT_204).end()
}
diff --git a/server/helpers/custom-validators/follows.ts b/server/helpers/custom-validators/follows.ts
index fbef7ad87..8f65552c3 100644
--- a/server/helpers/custom-validators/follows.ts
+++ b/server/helpers/custom-validators/follows.ts
@@ -1,4 +1,4 @@
-import { exists } from './misc'
+import { exists, isArray } from './misc'
import { FollowState } from '@shared/models'
function isFollowStateValid (value: FollowState) {
@@ -7,8 +7,24 @@ function isFollowStateValid (value: FollowState) {
return value === 'pending' || value === 'accepted'
}
+function isRemoteHandleValid (value: string) {
+ if (!exists(value)) return false
+ if (typeof value !== 'string') return false
+
+ return value.includes('@')
+}
+
+function isEachUniqueHandleValid (handles: string[]) {
+ return isArray(handles) &&
+ handles.every(handle => {
+ return isRemoteHandleValid(handle) && handles.indexOf(handle) === handles.lastIndexOf(handle)
+ })
+}
+
// ---------------------------------------------------------------------------
export {
- isFollowStateValid
+ isFollowStateValid,
+ isRemoteHandleValid,
+ isEachUniqueHandleValid
}
diff --git a/server/helpers/custom-validators/servers.ts b/server/helpers/custom-validators/servers.ts
index adf1ea497..c0f8b6aeb 100644
--- a/server/helpers/custom-validators/servers.ts
+++ b/server/helpers/custom-validators/servers.ts
@@ -19,7 +19,6 @@ function isHostValid (host: string) {
function isEachUniqueHostValid (hosts: string[]) {
return isArray(hosts) &&
- hosts.length !== 0 &&
hosts.every(host => {
return isHostValid(host) && hosts.indexOf(host) === hosts.lastIndexOf(host)
})
diff --git a/server/lib/activitypub/crawl.ts b/server/lib/activitypub/crawl.ts
index cd117f571..28ff5225a 100644
--- a/server/lib/activitypub/crawl.ts
+++ b/server/lib/activitypub/crawl.ts
@@ -1,3 +1,4 @@
+import { retryTransactionWrapper } from '@server/helpers/database-utils'
import * as Bluebird from 'bluebird'
import { URL } from 'url'
import { ActivityPubOrderedCollection } from '../../../shared/models/activitypub'
@@ -51,7 +52,7 @@ async function crawlCollectionPage
(argUrl: string, handler: HandlerFunction
}
}
- if (cleaner) await cleaner(startDate)
+ if (cleaner) await retryTransactionWrapper(cleaner, startDate)
}
export {
diff --git a/server/lib/activitypub/follow.ts b/server/lib/activitypub/follow.ts
index c1bd667e0..741b54df5 100644
--- a/server/lib/activitypub/follow.ts
+++ b/server/lib/activitypub/follow.ts
@@ -31,6 +31,21 @@ async function autoFollowBackIfNeeded (actorFollow: MActorFollowActors, transact
}
}
-export {
- autoFollowBackIfNeeded
+// If we only have an host, use a default account handle
+function getRemoteNameAndHost (handleOrHost: string) {
+ let name = SERVER_ACTOR_NAME
+ let host = handleOrHost
+
+ const splitted = handleOrHost.split('@')
+ if (splitted.length === 2) {
+ name = splitted[0]
+ host = splitted[1]
+ }
+
+ return { name, host }
+}
+
+export {
+ autoFollowBackIfNeeded,
+ getRemoteNameAndHost
}
diff --git a/server/models/actor/actor-follow.ts b/server/models/actor/actor-follow.ts
index 3a09e51d6..83c00a22d 100644
--- a/server/models/actor/actor-follow.ts
+++ b/server/models/actor/actor-follow.ts
@@ -324,13 +324,13 @@ export class ActorFollowModel extends Model s.follows)
// Get the access tokens
await setAccessTokensToServers(servers)
})
- it('Should not have followers', async function () {
- for (const server of servers) {
- const body = await server.follows.getFollowers({ start: 0, count: 5, sort: 'createdAt' })
- expect(body.total).to.equal(0)
+ describe('Data propagation after follow', function () {
- const follows = body.data
+ it('Should not have followers/followings', async function () {
+ for (const server of servers) {
+ const bodies = await Promise.all([
+ server.follows.getFollowers({ start: 0, count: 5, sort: 'createdAt' }),
+ server.follows.getFollowings({ start: 0, count: 5, sort: 'createdAt' })
+ ])
+
+ for (const body of bodies) {
+ expect(body.total).to.equal(0)
+
+ const follows = body.data
+ expect(follows).to.be.an('array')
+ expect(follows).to.have.lengthOf(0)
+ }
+ }
+ })
+
+ it('Should have server 1 following root account of server 2 and server 3', async function () {
+ this.timeout(30000)
+
+ await servers[0].follows.follow({
+ hosts: [ servers[2].url ],
+ handles: [ 'root@' + servers[1].host ]
+ })
+
+ await waitJobs(servers)
+ })
+
+ it('Should have 2 followings on server 1', async function () {
+ const body = await servers[0].follows.getFollowings({ start: 0, count: 1, sort: 'createdAt' })
+ expect(body.total).to.equal(2)
+
+ let follows = body.data
expect(follows).to.be.an('array')
- expect(follows.length).to.equal(0)
- }
- })
+ expect(follows).to.have.lengthOf(1)
- it('Should not have following', async function () {
- for (const server of servers) {
- const body = await server.follows.getFollowings({ start: 0, count: 5, sort: 'createdAt' })
- expect(body.total).to.equal(0)
+ const body2 = await servers[0].follows.getFollowings({ start: 1, count: 1, sort: 'createdAt' })
+ follows = follows.concat(body2.data)
- const follows = body.data
- expect(follows).to.be.an('array')
- expect(follows.length).to.equal(0)
- }
- })
+ const server2Follow = follows.find(f => f.following.host === servers[1].host)
+ const server3Follow = follows.find(f => f.following.host === servers[2].host)
- it('Should have server 1 following server 2 and 3', async function () {
- this.timeout(30000)
+ expect(server2Follow).to.not.be.undefined
+ expect(server2Follow.following.name).to.equal('root')
+ expect(server2Follow.state).to.equal('accepted')
- await followsCommands[0].follow({ targets: [ servers[1].url, servers[2].url ] })
+ expect(server3Follow).to.not.be.undefined
+ expect(server3Follow.following.name).to.equal('peertube')
+ expect(server3Follow.state).to.equal('accepted')
+ })
- await waitJobs(servers)
- })
-
- it('Should have 2 followings on server 1', async function () {
- const body = await followsCommands[0].getFollowings({ start: 0, count: 1, sort: 'createdAt' })
- expect(body.total).to.equal(2)
-
- let follows = body.data
- expect(follows).to.be.an('array')
- expect(follows.length).to.equal(1)
-
- const body2 = await followsCommands[0].getFollowings({ start: 1, count: 1, sort: 'createdAt' })
- follows = follows.concat(body2.data)
-
- const server2Follow = follows.find(f => f.following.host === 'localhost:' + servers[1].port)
- const server3Follow = follows.find(f => f.following.host === 'localhost:' + servers[2].port)
-
- expect(server2Follow).to.not.be.undefined
- expect(server3Follow).to.not.be.undefined
- expect(server2Follow.state).to.equal('accepted')
- expect(server3Follow.state).to.equal('accepted')
- })
-
- it('Should search/filter followings on server 1', async function () {
- const sort = 'createdAt'
- const start = 0
- const count = 1
-
- {
- const search = ':' + servers[1].port
-
- {
- const body = await followsCommands[0].getFollowings({ start, count, sort, search })
- expect(body.total).to.equal(1)
+ it('Should have 0 followings on server 2 and 3', async function () {
+ for (const server of [ servers[1], servers[2] ]) {
+ const body = await server.follows.getFollowings({ start: 0, count: 5, sort: 'createdAt' })
+ expect(body.total).to.equal(0)
const follows = body.data
- expect(follows.length).to.equal(1)
- expect(follows[0].following.host).to.equal('localhost:' + servers[1].port)
+ expect(follows).to.be.an('array')
+ expect(follows).to.have.lengthOf(0)
}
+ })
- {
- const body = await followsCommands[0].getFollowings({ start, count, sort, search, state: 'accepted' })
- expect(body.total).to.equal(1)
- expect(body.data).to.have.lengthOf(1)
- }
-
- {
- const body = await followsCommands[0].getFollowings({ start, count, sort, search, state: 'accepted', actorType: 'Person' })
- expect(body.total).to.equal(0)
- expect(body.data).to.have.lengthOf(0)
- }
-
- {
- const body = await followsCommands[0].getFollowings({
- start,
- count,
- sort,
- search,
- state: 'accepted',
- actorType: 'Application'
- })
- expect(body.total).to.equal(1)
- expect(body.data).to.have.lengthOf(1)
- }
-
- {
- const body = await followsCommands[0].getFollowings({ start, count, sort, search, state: 'pending' })
- expect(body.total).to.equal(0)
- expect(body.data).to.have.lengthOf(0)
- }
- }
-
- {
- const body = await followsCommands[0].getFollowings({ start, count, sort, search: 'bla' })
- expect(body.total).to.equal(0)
-
- expect(body.data.length).to.equal(0)
- }
- })
-
- it('Should have 0 followings on server 2 and 3', async function () {
- for (const server of [ servers[1], servers[2] ]) {
- const body = await server.follows.getFollowings({ start: 0, count: 5, sort: 'createdAt' })
- expect(body.total).to.equal(0)
-
- const follows = body.data
- expect(follows).to.be.an('array')
- expect(follows.length).to.equal(0)
- }
- })
-
- it('Should have 1 followers on server 2 and 3', async function () {
- for (const server of [ servers[1], servers[2] ]) {
- const body = await server.follows.getFollowers({ start: 0, count: 1, sort: 'createdAt' })
+ it('Should have 1 followers on server 3', async function () {
+ const body = await servers[2].follows.getFollowers({ start: 0, count: 1, sort: 'createdAt' })
expect(body.total).to.equal(1)
const follows = body.data
expect(follows).to.be.an('array')
- expect(follows.length).to.equal(1)
+ expect(follows).to.have.lengthOf(1)
expect(follows[0].follower.host).to.equal('localhost:' + servers[0].port)
- }
- })
+ })
- it('Should search/filter followers on server 2', async function () {
- const start = 0
- const count = 5
- const sort = 'createdAt'
-
- {
- const search = servers[0].port + ''
-
- {
- const body = await followsCommands[2].getFollowers({ start, count, sort, search })
- expect(body.total).to.equal(1)
+ it('Should have 0 followers on server 1 and 2', async function () {
+ for (const server of [ servers[0], servers[1] ]) {
+ const body = await server.follows.getFollowers({ start: 0, count: 5, sort: 'createdAt' })
+ expect(body.total).to.equal(0)
const follows = body.data
- expect(follows.length).to.equal(1)
- expect(follows[0].following.host).to.equal('localhost:' + servers[2].port)
+ expect(follows).to.be.an('array')
+ expect(follows).to.have.lengthOf(0)
+ }
+ })
+
+ it('Should search/filter followings on server 1', async function () {
+ const sort = 'createdAt'
+ const start = 0
+ const count = 1
+
+ {
+ const search = ':' + servers[1].port
+
+ {
+ const body = await servers[0].follows.getFollowings({ start, count, sort, search })
+ expect(body.total).to.equal(1)
+
+ const follows = body.data
+ expect(follows).to.have.lengthOf(1)
+ expect(follows[0].following.host).to.equal(servers[1].host)
+ }
+
+ {
+ const body = await servers[0].follows.getFollowings({ start, count, sort, search, state: 'accepted' })
+ expect(body.total).to.equal(1)
+ expect(body.data).to.have.lengthOf(1)
+ }
+
+ {
+ const body = await servers[0].follows.getFollowings({ start, count, sort, search, state: 'accepted', actorType: 'Person' })
+ expect(body.total).to.equal(1)
+ expect(body.data).to.have.lengthOf(1)
+ }
+
+ {
+ const body = await servers[0].follows.getFollowings({
+ start,
+ count,
+ sort,
+ search,
+ state: 'accepted',
+ actorType: 'Application'
+ })
+ expect(body.total).to.equal(0)
+ expect(body.data).to.have.lengthOf(0)
+ }
+
+ {
+ const body = await servers[0].follows.getFollowings({ start, count, sort, search, state: 'pending' })
+ expect(body.total).to.equal(0)
+ expect(body.data).to.have.lengthOf(0)
+ }
}
{
- const body = await followsCommands[2].getFollowers({ start, count, sort, search, state: 'accepted' })
+ const body = await servers[0].follows.getFollowings({ start, count, sort, search: 'root' })
expect(body.total).to.equal(1)
expect(body.data).to.have.lengthOf(1)
}
{
- const body = await followsCommands[2].getFollowers({ start, count, sort, search, state: 'accepted', actorType: 'Person' })
+ const body = await servers[0].follows.getFollowings({ start, count, sort, search: 'bla' })
expect(body.total).to.equal(0)
+
expect(body.data).to.have.lengthOf(0)
}
+ })
+
+ it('Should search/filter followers on server 2', async function () {
+ const start = 0
+ const count = 5
+ const sort = 'createdAt'
+
+ {
+ const search = servers[0].port + ''
+
+ {
+ const body = await servers[2].follows.getFollowers({ start, count, sort, search })
+ expect(body.total).to.equal(1)
+
+ const follows = body.data
+ expect(follows).to.have.lengthOf(1)
+ expect(follows[0].following.host).to.equal(servers[2].host)
+ }
+
+ {
+ const body = await servers[2].follows.getFollowers({ start, count, sort, search, state: 'accepted' })
+ expect(body.total).to.equal(1)
+ expect(body.data).to.have.lengthOf(1)
+ }
+
+ {
+ const body = await servers[2].follows.getFollowers({ start, count, sort, search, state: 'accepted', actorType: 'Person' })
+ expect(body.total).to.equal(0)
+ expect(body.data).to.have.lengthOf(0)
+ }
+
+ {
+ const body = await servers[2].follows.getFollowers({
+ start,
+ count,
+ sort,
+ search,
+ state: 'accepted',
+ actorType: 'Application'
+ })
+ expect(body.total).to.equal(1)
+ expect(body.data).to.have.lengthOf(1)
+ }
+
+ {
+ const body = await servers[2].follows.getFollowers({ start, count, sort, search, state: 'pending' })
+ expect(body.total).to.equal(0)
+ expect(body.data).to.have.lengthOf(0)
+ }
+ }
{
- const body = await followsCommands[2].getFollowers({
- start,
- count,
- sort,
- search,
- state: 'accepted',
- actorType: 'Application'
- })
- expect(body.total).to.equal(1)
- expect(body.data).to.have.lengthOf(1)
- }
-
- {
- const body = await followsCommands[2].getFollowers({ start, count, sort, search, state: 'pending' })
+ const body = await servers[2].follows.getFollowers({ start, count, sort, search: 'bla' })
expect(body.total).to.equal(0)
- expect(body.data).to.have.lengthOf(0)
- }
- }
- {
- const body = await followsCommands[2].getFollowers({ start, count, sort, search: 'bla' })
+ const follows = body.data
+ expect(follows).to.have.lengthOf(0)
+ }
+ })
+
+ it('Should have the correct follows counts', async function () {
+ await expectAccountFollows({ server: servers[0], handle: 'peertube@' + servers[0].host, followers: 0, following: 2 })
+ await expectAccountFollows({ server: servers[0], handle: 'root@' + servers[1].host, followers: 1, following: 0 })
+ await expectAccountFollows({ server: servers[0], handle: 'peertube@' + servers[2].host, followers: 1, following: 0 })
+
+ // Server 2 and 3 does not know server 1 follow another server (there was not a refresh)
+ await expectAccountFollows({ server: servers[1], handle: 'peertube@' + servers[0].host, followers: 0, following: 1 })
+ await expectAccountFollows({ server: servers[1], handle: 'root@' + servers[1].host, followers: 1, following: 0 })
+ await expectAccountFollows({ server: servers[1], handle: 'peertube@' + servers[1].host, followers: 0, following: 0 })
+
+ await expectAccountFollows({ server: servers[2], handle: 'peertube@' + servers[0].host, followers: 0, following: 1 })
+ await expectAccountFollows({ server: servers[2], handle: 'peertube@' + servers[2].host, followers: 1, following: 0 })
+ })
+
+ it('Should unfollow server 3 on server 1', async function () {
+ this.timeout(15000)
+
+ await servers[0].follows.unfollow({ target: servers[2] })
+
+ await waitJobs(servers)
+ })
+
+ it('Should not follow server 3 on server 1 anymore', async function () {
+ const body = await servers[0].follows.getFollowings({ start: 0, count: 2, sort: 'createdAt' })
+ expect(body.total).to.equal(1)
+
+ const follows = body.data
+ expect(follows).to.be.an('array')
+ expect(follows).to.have.lengthOf(1)
+
+ expect(follows[0].following.host).to.equal(servers[1].host)
+ })
+
+ it('Should not have server 1 as follower on server 3 anymore', async function () {
+ const body = await servers[2].follows.getFollowers({ start: 0, count: 1, sort: 'createdAt' })
expect(body.total).to.equal(0)
const follows = body.data
- expect(follows.length).to.equal(0)
- }
+ expect(follows).to.be.an('array')
+ expect(follows).to.have.lengthOf(0)
+ })
+
+ it('Should have the correct follows counts after the unfollow', async function () {
+ await expectAccountFollows({ server: servers[0], handle: 'peertube@' + servers[0].host, followers: 0, following: 1 })
+ await expectAccountFollows({ server: servers[0], handle: 'root@' + servers[1].host, followers: 1, following: 0 })
+ await expectAccountFollows({ server: servers[0], handle: 'peertube@' + servers[2].host, followers: 0, following: 0 })
+
+ await expectAccountFollows({ server: servers[1], handle: 'peertube@' + servers[0].host, followers: 0, following: 1 })
+ await expectAccountFollows({ server: servers[1], handle: 'root@' + servers[1].host, followers: 1, following: 0 })
+ await expectAccountFollows({ server: servers[1], handle: 'peertube@' + servers[1].host, followers: 0, following: 0 })
+
+ await expectAccountFollows({ server: servers[2], handle: 'peertube@' + servers[0].host, followers: 0, following: 0 })
+ await expectAccountFollows({ server: servers[2], handle: 'peertube@' + servers[2].host, followers: 0, following: 0 })
+ })
+
+ it('Should upload a video on server 2 and 3 and propagate only the video of server 2', async function () {
+ this.timeout(60000)
+
+ await servers[1].videos.upload({ attributes: { name: 'server2' } })
+ await servers[2].videos.upload({ attributes: { name: 'server3' } })
+
+ await waitJobs(servers)
+
+ {
+ const { total, data } = await servers[0].videos.list()
+ expect(total).to.equal(1)
+ expect(data[0].name).to.equal('server2')
+ }
+
+ {
+ const { total, data } = await servers[1].videos.list()
+ expect(total).to.equal(1)
+ expect(data[0].name).to.equal('server2')
+ }
+
+ {
+ const { total, data } = await servers[2].videos.list()
+ expect(total).to.equal(1)
+ expect(data[0].name).to.equal('server3')
+ }
+ })
+
+ it('Should remove account follow', async function () {
+ this.timeout(15000)
+
+ await servers[0].follows.unfollow({ target: 'root@' + servers[1].host })
+
+ await waitJobs(servers)
+ })
+
+ it('Should have removed the account follow', async function () {
+ await expectAccountFollows({ server: servers[0], handle: 'root@' + servers[1].host, followers: 0, following: 0 })
+ await expectAccountFollows({ server: servers[1], handle: 'root@' + servers[1].host, followers: 0, following: 0 })
+
+ {
+ const { total, data } = await servers[0].follows.getFollowings()
+ expect(total).to.equal(0)
+ expect(data).to.have.lengthOf(0)
+ }
+
+ {
+ const { total, data } = await servers[0].videos.list()
+ expect(total).to.equal(0)
+ expect(data).to.have.lengthOf(0)
+ }
+ })
+
+ it('Should follow a channel', async function () {
+ this.timeout(15000)
+
+ await servers[0].follows.follow({
+ handles: [ 'root_channel@' + servers[1].host ]
+ })
+
+ await waitJobs(servers)
+
+ await expectChannelsFollows({ server: servers[0], handle: 'root_channel@' + servers[1].host, followers: 1, following: 0 })
+ await expectChannelsFollows({ server: servers[1], handle: 'root_channel@' + servers[1].host, followers: 1, following: 0 })
+
+ {
+ const { total, data } = await servers[0].follows.getFollowings()
+ expect(total).to.equal(1)
+ expect(data).to.have.lengthOf(1)
+ }
+
+ {
+ const { total, data } = await servers[0].videos.list()
+ expect(total).to.equal(1)
+ expect(data).to.have.lengthOf(1)
+ }
+ })
})
- it('Should have 0 followers on server 1', async function () {
- const body = await followsCommands[0].getFollowers({ start: 0, count: 5, sort: 'createdAt' })
- expect(body.total).to.equal(0)
-
- const follows = body.data
- expect(follows).to.be.an('array')
- expect(follows.length).to.equal(0)
- })
-
- it('Should have the correct follows counts', async function () {
- await expectAccountFollows({ server: servers[0], handle: 'peertube@localhost:' + servers[0].port, followers: 0, following: 2 })
- await expectAccountFollows({ server: servers[0], handle: 'peertube@localhost:' + servers[1].port, followers: 1, following: 0 })
- await expectAccountFollows({ server: servers[0], handle: 'peertube@localhost:' + servers[2].port, followers: 1, following: 0 })
-
- // Server 2 and 3 does not know server 1 follow another server (there was not a refresh)
- await expectAccountFollows({ server: servers[1], handle: 'peertube@localhost:' + servers[0].port, followers: 0, following: 1 })
- await expectAccountFollows({ server: servers[1], handle: 'peertube@localhost:' + servers[1].port, followers: 1, following: 0 })
-
- await expectAccountFollows({ server: servers[2], handle: 'peertube@localhost:' + servers[0].port, followers: 0, following: 1 })
- await expectAccountFollows({ server: servers[2], handle: 'peertube@localhost:' + servers[2].port, followers: 1, following: 0 })
- })
-
- it('Should unfollow server 3 on server 1', async function () {
- this.timeout(5000)
-
- await followsCommands[0].unfollow({ target: servers[2] })
-
- await waitJobs(servers)
- })
-
- it('Should not follow server 3 on server 1 anymore', async function () {
- const body = await followsCommands[0].getFollowings({ start: 0, count: 2, sort: 'createdAt' })
- expect(body.total).to.equal(1)
-
- const follows = body.data
- expect(follows).to.be.an('array')
- expect(follows.length).to.equal(1)
-
- expect(follows[0].following.host).to.equal('localhost:' + servers[1].port)
- })
-
- it('Should not have server 1 as follower on server 3 anymore', async function () {
- const body = await followsCommands[2].getFollowers({ start: 0, count: 1, sort: 'createdAt' })
- expect(body.total).to.equal(0)
-
- const follows = body.data
- expect(follows).to.be.an('array')
- expect(follows.length).to.equal(0)
- })
-
- it('Should have the correct follows counts 2', async function () {
- await expectAccountFollows({ server: servers[0], handle: 'peertube@localhost:' + servers[0].port, followers: 0, following: 1 })
- await expectAccountFollows({ server: servers[0], handle: 'peertube@localhost:' + servers[1].port, followers: 1, following: 0 })
-
- await expectAccountFollows({ server: servers[1], handle: 'peertube@localhost:' + servers[0].port, followers: 0, following: 1 })
- await expectAccountFollows({ server: servers[1], handle: 'peertube@localhost:' + servers[1].port, followers: 1, following: 0 })
-
- await expectAccountFollows({ server: servers[2], handle: 'peertube@localhost:' + servers[0].port, followers: 0, following: 0 })
- await expectAccountFollows({ server: servers[2], handle: 'peertube@localhost:' + servers[2].port, followers: 0, following: 0 })
- })
-
- it('Should upload a video on server 2 and 3 and propagate only the video of server 2', async function () {
- this.timeout(60000)
-
- await servers[1].videos.upload({ attributes: { name: 'server2' } })
- await servers[2].videos.upload({ attributes: { name: 'server3' } })
-
- await waitJobs(servers)
-
- {
- const { total, data } = await servers[0].videos.list()
- expect(total).to.equal(1)
- expect(data[0].name).to.equal('server2')
- }
-
- {
- const { total, data } = await servers[1].videos.list()
- expect(total).to.equal(1)
- expect(data[0].name).to.equal('server2')
- }
-
- {
- const { total, data } = await servers[2].videos.list()
- expect(total).to.equal(1)
- expect(data[0].name).to.equal('server3')
- }
- })
-
- describe('Should propagate data on a new following', function () {
- let video4: Video
+ describe('Should propagate data on a new server follow', function () {
+ let video4: VideoCreateResult
before(async function () {
this.timeout(50000)
@@ -324,83 +385,64 @@ describe('Test follows', function () {
await servers[2].videos.upload({ attributes: { name: 'server3-2' } })
await servers[2].videos.upload({ attributes: { name: 'server3-3' } })
- await servers[2].videos.upload({ attributes: video4Attributes })
+ video4 = await servers[2].videos.upload({ attributes: video4Attributes })
await servers[2].videos.upload({ attributes: { name: 'server3-5' } })
await servers[2].videos.upload({ attributes: { name: 'server3-6' } })
{
const userAccessToken = await servers[2].users.generateUserAndToken('captain')
- const { data } = await servers[2].videos.list()
- video4 = data.find(v => v.name === 'server3-4')
-
- {
- await servers[2].videos.rate({ id: video4.id, rating: 'like' })
- await servers[2].videos.rate({ token: userAccessToken, id: video4.id, rating: 'dislike' })
- }
-
- {
- {
- const text = 'my super first comment'
- const created = await servers[2].comments.createThread({ videoId: video4.id, text })
- const threadId = created.id
-
- const text1 = 'my super answer to thread 1'
- const childComment = await servers[2].comments.addReply({ videoId: video4.id, toCommentId: threadId, text: text1 })
-
- const text2 = 'my super answer to answer of thread 1'
- await servers[2].comments.addReply({ videoId: video4.id, toCommentId: childComment.id, text: text2 })
-
- const text3 = 'my second answer to thread 1'
- await servers[2].comments.addReply({ videoId: video4.id, toCommentId: threadId, text: text3 })
- }
-
- {
- const text = 'will be deleted'
- const created = await servers[2].comments.createThread({ videoId: video4.id, text })
- const threadId = created.id
-
- const text1 = 'answer to deleted'
- await servers[2].comments.addReply({ videoId: video4.id, toCommentId: threadId, text: text1 })
-
- const text2 = 'will also be deleted'
- const childComment = await servers[2].comments.addReply({ videoId: video4.id, toCommentId: threadId, text: text2 })
-
- const text3 = 'my second answer to deleted'
- await servers[2].comments.addReply({ videoId: video4.id, toCommentId: childComment.id, text: text3 })
-
- await servers[2].comments.delete({ videoId: video4.id, commentId: threadId })
- await servers[2].comments.delete({ videoId: video4.id, commentId: childComment.id })
- }
- }
-
- {
- await servers[2].captions.createVideoCaption({
- language: 'ar',
- videoId: video4.id,
- fixture: 'subtitle-good2.vtt'
- })
- }
+ await servers[2].videos.rate({ id: video4.id, rating: 'like' })
+ await servers[2].videos.rate({ token: userAccessToken, id: video4.id, rating: 'dislike' })
}
+ {
+ await servers[2].comments.createThread({ videoId: video4.id, text: 'my super first comment' })
+
+ await servers[2].comments.addReplyToLastThread({ text: 'my super answer to thread 1' })
+ await servers[2].comments.addReplyToLastReply({ text: 'my super answer to answer of thread 1' })
+ await servers[2].comments.addReplyToLastThread({ text: 'my second answer to thread 1' })
+ }
+
+ {
+ const { id: threadId } = await servers[2].comments.createThread({ videoId: video4.id, text: 'will be deleted' })
+ await servers[2].comments.addReplyToLastThread({ text: 'answer to deleted' })
+
+ const { id: replyId } = await servers[2].comments.addReplyToLastThread({ text: 'will also be deleted' })
+
+ await servers[2].comments.addReplyToLastReply({ text: 'my second answer to deleted' })
+
+ await servers[2].comments.delete({ videoId: video4.id, commentId: threadId })
+ await servers[2].comments.delete({ videoId: video4.id, commentId: replyId })
+ }
+
+ await servers[2].captions.createVideoCaption({
+ language: 'ar',
+ videoId: video4.id,
+ fixture: 'subtitle-good2.vtt'
+ })
+
await waitJobs(servers)
// Server 1 follows server 3
- await followsCommands[0].follow({ targets: [ servers[2].url ] })
+ await servers[0].follows.follow({ hosts: [ servers[2].url ] })
await waitJobs(servers)
})
- it('Should have the correct follows counts 3', async function () {
- await expectAccountFollows({ server: servers[0], handle: 'peertube@localhost:' + servers[0].port, followers: 0, following: 2 })
- await expectAccountFollows({ server: servers[0], handle: 'peertube@localhost:' + servers[1].port, followers: 1, following: 0 })
- await expectAccountFollows({ server: servers[0], handle: 'peertube@localhost:' + servers[2].port, followers: 1, following: 0 })
+ it('Should have the correct follows counts', async function () {
+ await expectAccountFollows({ server: servers[0], handle: 'peertube@' + servers[0].host, followers: 0, following: 2 })
+ await expectAccountFollows({ server: servers[0], handle: 'root@' + servers[1].host, followers: 0, following: 0 })
+ await expectChannelsFollows({ server: servers[0], handle: 'root_channel@' + servers[1].host, followers: 1, following: 0 })
+ await expectAccountFollows({ server: servers[0], handle: 'peertube@' + servers[2].host, followers: 1, following: 0 })
- await expectAccountFollows({ server: servers[1], handle: 'peertube@localhost:' + servers[0].port, followers: 0, following: 1 })
- await expectAccountFollows({ server: servers[1], handle: 'peertube@localhost:' + servers[1].port, followers: 1, following: 0 })
+ await expectAccountFollows({ server: servers[1], handle: 'peertube@' + servers[0].host, followers: 0, following: 1 })
+ await expectAccountFollows({ server: servers[1], handle: 'peertube@' + servers[1].host, followers: 0, following: 0 })
+ await expectAccountFollows({ server: servers[1], handle: 'root@' + servers[1].host, followers: 0, following: 0 })
+ await expectChannelsFollows({ server: servers[1], handle: 'root_channel@' + servers[1].host, followers: 1, following: 0 })
- await expectAccountFollows({ server: servers[2], handle: 'peertube@localhost:' + servers[0].port, followers: 0, following: 1 })
- await expectAccountFollows({ server: servers[2], handle: 'peertube@localhost:' + servers[2].port, followers: 1, following: 0 })
+ await expectAccountFollows({ server: servers[2], handle: 'peertube@' + servers[0].host, followers: 0, following: 1 })
+ await expectAccountFollows({ server: servers[2], handle: 'peertube@' + servers[2].host, followers: 1, following: 0 })
})
it('Should have propagated videos', async function () {
@@ -426,7 +468,7 @@ describe('Test follows', function () {
support: 'my super support text',
account: {
name: 'root',
- host: 'localhost:' + servers[2].port
+ host: servers[2].host
},
isLocal,
commentsEnabled: true,
@@ -467,7 +509,7 @@ describe('Test follows', function () {
expect(comment.videoId).to.equal(video4.id)
expect(comment.id).to.equal(comment.threadId)
expect(comment.account.name).to.equal('root')
- expect(comment.account.host).to.equal('localhost:' + servers[2].port)
+ expect(comment.account.host).to.equal(servers[2].host)
expect(comment.totalReplies).to.equal(3)
expect(dateIsValid(comment.createdAt as string)).to.be.true
expect(dateIsValid(comment.updatedAt as string)).to.be.true
@@ -541,14 +583,39 @@ describe('Test follows', function () {
it('Should unfollow server 3 on server 1 and does not list server 3 videos', async function () {
this.timeout(5000)
- await followsCommands[0].unfollow({ target: servers[2] })
+ await servers[0].follows.unfollow({ target: servers[2] })
await waitJobs(servers)
const { total } = await servers[0].videos.list()
expect(total).to.equal(1)
})
+ })
+ describe('Should propagate data on a new channel follow', function () {
+
+ before(async function () {
+ this.timeout(60000)
+
+ await servers[2].videos.upload({ attributes: { name: 'server3-7' } })
+
+ await waitJobs(servers)
+
+ const video = await servers[0].videos.find({ name: 'server3-7' })
+ expect(video).to.not.exist
+ })
+
+ it('Should have propagated channel video', async function () {
+ this.timeout(60000)
+
+ await servers[0].follows.follow({ handles: [ 'root_channel@' + servers[2].host ] })
+
+ await waitJobs(servers)
+
+ const video = await servers[0].videos.find({ name: 'server3-7' })
+
+ expect(video).to.exist
+ })
})
after(async function () {
diff --git a/server/tests/api/server/handle-down.ts b/server/tests/api/server/handle-down.ts
index 1f751c957..2f3950354 100644
--- a/server/tests/api/server/handle-down.ts
+++ b/server/tests/api/server/handle-down.ts
@@ -97,8 +97,8 @@ describe('Test handle downs', function () {
this.timeout(240000)
// Server 2 and 3 follow server 1
- await servers[1].follows.follow({ targets: [ servers[0].url ] })
- await servers[2].follows.follow({ targets: [ servers[0].url ] })
+ await servers[1].follows.follow({ hosts: [ servers[0].url ] })
+ await servers[2].follows.follow({ hosts: [ servers[0].url ] })
await waitJobs(servers)
@@ -180,7 +180,7 @@ describe('Test handle downs', function () {
await servers[1].follows.unfollow({ target: servers[0] })
await waitJobs(servers)
- await servers[1].follows.follow({ targets: [ servers[0].url ] })
+ await servers[1].follows.follow({ hosts: [ servers[0].url ] })
await waitJobs(servers)
diff --git a/server/tests/api/server/stats.ts b/server/tests/api/server/stats.ts
index 942602b70..5ec771429 100644
--- a/server/tests/api/server/stats.ts
+++ b/server/tests/api/server/stats.ts
@@ -43,7 +43,7 @@ describe('Test stats (excluding redundancy)', function () {
// Wait the video views repeatable job
await wait(8000)
- await servers[2].follows.follow({ targets: [ servers[0].url ] })
+ await servers[2].follows.follow({ hosts: [ servers[0].url ] })
await waitJobs(servers)
})
diff --git a/server/tests/api/users/user-subscriptions.ts b/server/tests/api/users/user-subscriptions.ts
index 565b4bd77..77b99886d 100644
--- a/server/tests/api/users/user-subscriptions.ts
+++ b/server/tests/api/users/user-subscriptions.ts
@@ -224,7 +224,7 @@ describe('Test users subscriptions', function () {
it('Should have server 1 follow server 3 and display server 3 videos', async function () {
this.timeout(60000)
- await servers[0].follows.follow({ targets: [ servers[2].url ] })
+ await servers[0].follows.follow({ hosts: [ servers[2].url ] })
await waitJobs(servers)
diff --git a/server/tests/api/users/users.ts b/server/tests/api/users/users.ts
index 066da88ee..1419ae820 100644
--- a/server/tests/api/users/users.ts
+++ b/server/tests/api/users/users.ts
@@ -103,7 +103,7 @@ describe('Test users', function () {
token = 'my_super_token'
await server.follows.follow({
- targets: [ 'http://example.com' ],
+ hosts: [ 'http://example.com' ],
token,
expectedStatus: HttpStatusCode.UNAUTHORIZED_401
})
diff --git a/shared/extra-utils/server/follows-command.ts b/shared/extra-utils/server/follows-command.ts
index dce674ac5..2b889cf66 100644
--- a/shared/extra-utils/server/follows-command.ts
+++ b/shared/extra-utils/server/follows-command.ts
@@ -1,5 +1,5 @@
import { pick } from 'lodash'
-import { ActivityPubActorType, ActorFollow, FollowState, HttpStatusCode, ResultList } from '@shared/models'
+import { ActivityPubActorType, ActorFollow, FollowState, HttpStatusCode, ResultList, ServerFollowCreate } from '@shared/models'
import { AbstractCommand, OverrideCommandOptions } from '../shared'
import { PeerTubeServer } from './server'
@@ -29,13 +29,13 @@ export class FollowsCommand extends AbstractCommand {
}
getFollowings (options: OverrideCommandOptions & {
- start: number
- count: number
- sort: string
+ start?: number
+ count?: number
+ sort?: string
search?: string
actorType?: ActivityPubActorType
state?: FollowState
- }) {
+ } = {}) {
const path = '/api/v1/server/following'
const toPick = [ 'start', 'count', 'sort', 'search', 'state', 'actorType' ]
@@ -52,26 +52,41 @@ export class FollowsCommand extends AbstractCommand {
}
follow (options: OverrideCommandOptions & {
- targets: string[]
+ hosts?: string[]
+ handles?: string[]
}) {
const path = '/api/v1/server/following'
- const hosts = options.targets.map(f => f.replace(/^http:\/\//, ''))
+ const fields: ServerFollowCreate = {}
+
+ if (options.hosts) {
+ fields.hosts = options.hosts.map(f => f.replace(/^http:\/\//, ''))
+ }
+
+ if (options.handles) {
+ fields.handles = options.handles
+ }
return this.postBodyRequest({
...options,
path,
- fields: { hosts },
+ fields,
implicitToken: true,
defaultExpectedStatus: HttpStatusCode.NO_CONTENT_204
})
}
async unfollow (options: OverrideCommandOptions & {
- target: PeerTubeServer
+ target: PeerTubeServer | string
}) {
- const path = '/api/v1/server/following/' + options.target.host
+ const { target } = options
+
+ const handle = typeof target === 'string'
+ ? target
+ : target.host
+
+ const path = '/api/v1/server/following/' + handle
return this.deleteRequest({
...options,
diff --git a/shared/extra-utils/server/follows.ts b/shared/extra-utils/server/follows.ts
index 0188be1aa..698238f29 100644
--- a/shared/extra-utils/server/follows.ts
+++ b/shared/extra-utils/server/follows.ts
@@ -3,8 +3,8 @@ import { PeerTubeServer } from './server'
async function doubleFollow (server1: PeerTubeServer, server2: PeerTubeServer) {
await Promise.all([
- server1.follows.follow({ targets: [ server2.url ] }),
- server2.follows.follow({ targets: [ server1.url ] })
+ server1.follows.follow({ hosts: [ server2.url ] }),
+ server2.follows.follow({ hosts: [ server1.url ] })
])
// Wait request propagation
diff --git a/shared/extra-utils/users/accounts.ts b/shared/extra-utils/users/actors.ts
similarity index 50%
rename from shared/extra-utils/users/accounts.ts
rename to shared/extra-utils/users/actors.ts
index 9fc1bcfc4..cfcc7d0a7 100644
--- a/shared/extra-utils/users/accounts.ts
+++ b/shared/extra-utils/users/actors.ts
@@ -4,22 +4,31 @@ import { expect } from 'chai'
import { pathExists, readdir } from 'fs-extra'
import { join } from 'path'
import { root } from '@server/helpers/core-utils'
+import { Account, VideoChannel } from '@shared/models'
import { PeerTubeServer } from '../server'
+async function expectChannelsFollows (options: {
+ server: PeerTubeServer
+ handle: string
+ followers: number
+ following: number
+}) {
+ const { server } = options
+ const { data } = await server.channels.list()
+
+ return expectActorFollow({ ...options, data })
+}
+
async function expectAccountFollows (options: {
server: PeerTubeServer
handle: string
followers: number
following: number
}) {
- const { server, handle, followers, following } = options
+ const { server } = options
+ const { data } = await server.accounts.list()
- const body = await server.accounts.list()
- const account = body.data.find(a => a.name + '@' + a.host === handle)
-
- const message = `${handle} on ${server.url}`
- expect(account.followersCount).to.equal(followers, message)
- expect(account.followingCount).to.equal(following, message)
+ return expectActorFollow({ ...options, data })
}
async function checkActorFilesWereRemoved (filename: string, serverNumber: number) {
@@ -40,5 +49,25 @@ async function checkActorFilesWereRemoved (filename: string, serverNumber: numbe
export {
expectAccountFollows,
+ expectChannelsFollows,
checkActorFilesWereRemoved
}
+
+// ---------------------------------------------------------------------------
+
+function expectActorFollow (options: {
+ server: PeerTubeServer
+ data: (Account | VideoChannel)[]
+ handle: string
+ followers: number
+ following: number
+}) {
+ const { server, data, handle, followers, following } = options
+
+ const actor = data.find(a => a.name + '@' + a.host === handle)
+ const message = `${handle} on ${server.url}`
+
+ expect(actor, message).to.exist
+ expect(actor.followersCount).to.equal(followers, message)
+ expect(actor.followingCount).to.equal(following, message)
+}
diff --git a/shared/extra-utils/users/index.ts b/shared/extra-utils/users/index.ts
index fbb454e8f..460a06f70 100644
--- a/shared/extra-utils/users/index.ts
+++ b/shared/extra-utils/users/index.ts
@@ -1,5 +1,5 @@
export * from './accounts-command'
-export * from './accounts'
+export * from './actors'
export * from './blocklist-command'
export * from './login'
export * from './login-command'
diff --git a/shared/extra-utils/videos/comments-command.ts b/shared/extra-utils/videos/comments-command.ts
index dd14e4b64..5034c57ad 100644
--- a/shared/extra-utils/videos/comments-command.ts
+++ b/shared/extra-utils/videos/comments-command.ts
@@ -5,6 +5,10 @@ import { AbstractCommand, OverrideCommandOptions } from '../shared'
export class CommentsCommand extends AbstractCommand {
+ private lastVideoId: number | string
+ private lastThreadId: number
+ private lastReplyId: number
+
listForAdmin (options: OverrideCommandOptions & {
start?: number
count?: number
@@ -80,6 +84,9 @@ export class CommentsCommand extends AbstractCommand {
defaultExpectedStatus: HttpStatusCode.OK_200
}))
+ this.lastThreadId = body.comment.id
+ this.lastVideoId = videoId
+
return body.comment
}
@@ -100,9 +107,23 @@ export class CommentsCommand extends AbstractCommand {
defaultExpectedStatus: HttpStatusCode.OK_200
}))
+ this.lastReplyId = body.comment.id
+
return body.comment
}
+ async addReplyToLastReply (options: OverrideCommandOptions & {
+ text: string
+ }) {
+ return this.addReply({ ...options, videoId: this.lastVideoId, toCommentId: this.lastReplyId })
+ }
+
+ async addReplyToLastThread (options: OverrideCommandOptions & {
+ text: string
+ }) {
+ return this.addReply({ ...options, videoId: this.lastVideoId, toCommentId: this.lastThreadId })
+ }
+
async findCommentId (options: OverrideCommandOptions & {
videoId: number | string
text: string
diff --git a/shared/extra-utils/videos/videos-command.ts b/shared/extra-utils/videos/videos-command.ts
index 40cc4dc28..98465e8f6 100644
--- a/shared/extra-utils/videos/videos-command.ts
+++ b/shared/extra-utils/videos/videos-command.ts
@@ -267,6 +267,16 @@ export class VideosCommand extends AbstractCommand {
// ---------------------------------------------------------------------------
+ async find (options: OverrideCommandOptions & {
+ name: string
+ }) {
+ const { data } = await this.list(options)
+
+ return data.find(v => v.name === options.name)
+ }
+
+ // ---------------------------------------------------------------------------
+
update (options: OverrideCommandOptions & {
id: number | string
attributes?: VideoEdit
diff --git a/shared/models/server/index.ts b/shared/models/server/index.ts
index 06bf5c599..0f7646c7a 100644
--- a/shared/models/server/index.ts
+++ b/shared/models/server/index.ts
@@ -10,4 +10,5 @@ export * from './peertube-problem-document.model'
export * from './server-config.model'
export * from './server-debug.model'
export * from './server-error-code.enum'
+export * from './server-follow-create.model'
export * from './server-stats.model'
diff --git a/shared/models/server/server-follow-create.model.ts b/shared/models/server/server-follow-create.model.ts
new file mode 100644
index 000000000..3f90c7d6f
--- /dev/null
+++ b/shared/models/server/server-follow-create.model.ts
@@ -0,0 +1,4 @@
+export interface ServerFollowCreate {
+ hosts?: string[]
+ handles?: string[]
+}
diff --git a/support/doc/api/openapi.yaml b/support/doc/api/openapi.yaml
index 99a725ead..76e78fe53 100644
--- a/support/doc/api/openapi.yaml
+++ b/support/doc/api/openapi.yaml
@@ -716,7 +716,7 @@ paths:
- admin
tags:
- Instance Follows
- summary: Follow a list of servers
+ summary: Follow a list of actors (PeerTube instance, channel or account)
responses:
'204':
description: successful operation
@@ -734,28 +734,32 @@ paths:
type: string
format: hostname
uniqueItems: true
+ handles:
+ type: array
+ items:
+ type: string
+ uniqueItems: true
- '/server/following/{host}':
+ '/server/following/{hostOrHandle}':
delete:
- summary: Unfollow a server
+ summary: Unfollow an actor (PeerTube instance, channel or account)
security:
- OAuth2:
- admin
tags:
- Instance Follows
parameters:
- - name: host
+ - name: hostOrHandle
in: path
required: true
- description: The host to unfollow
+ description: The hostOrHandle to unfollow
schema:
type: string
- format: hostname
responses:
'204':
description: successful operation
'404':
- description: host not found
+ description: host or handle not found
/users:
post:
diff --git a/server/middlewares/validators/follows.ts b/server/middlewares/validators/follows.ts
index 05cc66c38..16abdd096 100644
--- a/server/middlewares/validators/follows.ts
+++ b/server/middlewares/validators/follows.ts
@@ -1,7 +1,8 @@
import * as express from 'express'
import { body, param, query } from 'express-validator'
-import { isFollowStateValid } from '@server/helpers/custom-validators/follows'
+import { isEachUniqueHandleValid, isFollowStateValid, isRemoteHandleValid } from '@server/helpers/custom-validators/follows'
import { loadActorUrlOrGetFromWebfinger } from '@server/lib/activitypub/actors'
+import { getRemoteNameAndHost } from '@server/lib/activitypub/follow'
import { getServerActor } from '@server/models/application/application'
import { MActorFollowActorsDefault } from '@server/types/models'
import { HttpStatusCode } from '../../../shared/models/http/http-error-codes'
@@ -9,10 +10,11 @@ import { isTestInstance } from '../../helpers/core-utils'
import { isActorTypeValid, isValidActorHandle } from '../../helpers/custom-validators/activitypub/actor'
import { isEachUniqueHostValid, isHostValid } from '../../helpers/custom-validators/servers'
import { logger } from '../../helpers/logger'
-import { SERVER_ACTOR_NAME, WEBSERVER } from '../../initializers/constants'
+import { WEBSERVER } from '../../initializers/constants'
import { ActorModel } from '../../models/actor/actor'
import { ActorFollowModel } from '../../models/actor/actor-follow'
import { areValidationErrors } from './shared'
+import { ServerFollowCreate } from '@shared/models'
const listFollowsValidator = [
query('state')
@@ -30,29 +32,46 @@ const listFollowsValidator = [
]
const followValidator = [
- body('hosts').custom(isEachUniqueHostValid).withMessage('Should have an array of unique hosts'),
+ body('hosts')
+ .toArray()
+ .custom(isEachUniqueHostValid).withMessage('Should have an array of unique hosts'),
+
+ body('handles')
+ .toArray()
+ .custom(isEachUniqueHandleValid).withMessage('Should have an array of handles'),
(req: express.Request, res: express.Response, next: express.NextFunction) => {
// Force https if the administrator wants to make friends
if (isTestInstance() === false && WEBSERVER.SCHEME === 'http') {
return res
.status(HttpStatusCode.INTERNAL_SERVER_ERROR_500)
.json({
error: 'Cannot follow on a non HTTPS web server.'
})
- .end()
}
logger.debug('Checking follow parameters', { parameters: req.body })
if (areValidationErrors(req, res)) return
+ const body: ServerFollowCreate = req.body
+ if (body.hosts.length === 0 && body.handles.length === 0) {
+
+ return res
+ .status(HttpStatusCode.BAD_REQUEST_400)
+ .json({
+ error: 'You must provide at least one handle or one host.'
+ })
+ }
+
return next()
}
]
const removeFollowingValidator = [
- param('host').custom(isHostValid).withMessage('Should have a valid host'),
+ param('hostOrHandle')
+ .custom(value => isHostValid(value) || isRemoteHandleValid(value))
+ .withMessage('Should have a valid host/handle'),
async (req: express.Request, res: express.Response, next: express.NextFunction) => {
logger.debug('Checking unfollowing parameters', { parameters: req.params })
@@ -60,12 +79,14 @@ const removeFollowingValidator = [
if (areValidationErrors(req, res)) return
const serverActor = await getServerActor()
- const follow = await ActorFollowModel.loadByActorAndTargetNameAndHostForAPI(serverActor.id, SERVER_ACTOR_NAME, req.params.host)
+
+ const { name, host } = getRemoteNameAndHost(req.params.hostOrHandle)
+ const follow = await ActorFollowModel.loadByActorAndTargetNameAndHostForAPI(serverActor.id, name, host)
if (!follow) {
return res.fail({
status: HttpStatusCode.NOT_FOUND_404,
- message: `Following ${req.params.host} not found.`
+ message: `Follow ${req.params.hostOrHandle} not found.`
})
}
@@ -30,29 +32,46 @@ const listFollowsValidator = [
(req: express.Request, res: express.Response, next: express.NextFunction) => {
- // Force https if the administrator wants to make friends
+ // Force https if the administrator wants to follow remote actors
if (isTestInstance() === false && WEBSERVER.SCHEME === 'http') {
return res